@@ -1,5 +1,5 @@
 # mysql/base.py
-# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -800,12 +800,15 @@ from ...util import topological
 
 RESERVED_WORDS = set(
     [
+        "accessible",
         "accessible",
         "add",
+        "admin",
         "all",
         "alter",
         "analyze",
         "and",
+        "array",  # 8.0
         "as",
         "asc",
         "asensitive",
@@ -825,12 +828,14 @@ RESERVED_WORDS = set(
         "check",
         "collate",
         "column",
+        "columns",
         "condition",
         "constraint",
         "continue",
         "convert",
         "create",
         "cross",
+        "cume_dist",
         "current_date",
         "current_time",
         "current_timestamp",
@@ -860,13 +865,17 @@ RESERVED_WORDS = set(
         "each",
         "else",
         "elseif",
+        "empty",
         "enclosed",
         "escaped",
+        "except",
         "exists",
         "exit",
         "explain",
         "false",
         "fetch",
+        "fields",
+        "first_value",
         "float",
         "float4",
         "float8",
@@ -875,8 +884,14 @@ RESERVED_WORDS = set(
         "foreign",
         "from",
         "fulltext",
+        "function",
+        "general",
+        "generated",
+        "get",
         "grant",
         "group",
+        "grouping",
+        "groups",
         "having",
         "high_priority",
         "hour_microsecond",
@@ -884,6 +899,7 @@ RESERVED_WORDS = set(
         "hour_second",
         "if",
         "ignore",
+        "ignore_server_ids",
         "in",
         "index",
         "infile",
@@ -900,18 +916,23 @@ RESERVED_WORDS = set(
         "integer",
         "interval",
         "into",
+        "io_after_gtids",
+        "io_before_gtids",
         "is",
         "iterate",
         "join",
+        "json_table",
         "key",
         "keys",
         "kill",
+        "last_value",
         "leading",
         "leave",
         "left",
         "like",
         "limit",
         "linear",
+        "linear",
         "lines",
         "load",
         "localtime",
@@ -922,23 +943,33 @@ RESERVED_WORDS = set(
         "longtext",
         "loop",
         "low_priority",
+        "master_bind",
+        "master_heartbeat_period",
+        "master_ssl_verify_server_cert",
         "master_ssl_verify_server_cert",
         "match",
+        "maxvalue",
         "mediumblob",
         "mediumint",
         "mediumtext",
+        "member",  # 8.0
         "middleint",
         "minute_microsecond",
         "minute_second",
         "mod",
         "modifies",
         "natural",
-        "not",
         "no_write_to_binlog",
+        "not",
+        "nth_value",
+        "ntile",
         "null",
         "numeric",
+        "of",
         "on",
+        "one_shot",
         "optimize",
+        "optimizer_costs",
         "option",
         "optionally",
         "or",
@@ -946,16 +977,27 @@ RESERVED_WORDS = set(
         "out",
         "outer",
         "outfile",
+        "over",
+        "partition",
+        "percent_rank",
+        "persist",
+        "persist_only",
         "precision",
         "primary",
+        "privileges",
         "procedure",
         "purge",
         "range",
+        "range",
+        "rank",
         "read",
-        "reads",
+        "read_only",
         "read_only",
         "read_write",
+        "read_write",  # 5.1
+        "reads",
         "real",
+        "recursive",
         "references",
         "regexp",
         "release",
@@ -963,11 +1005,16 @@ RESERVED_WORDS = set(
         "repeat",
         "replace",
         "require",
+        "resignal",
         "restrict",
         "return",
         "revoke",
         "right",
         "rlike",
+        "role",
+        "row",
+        "row_number",
+        "rows",
         "schema",
         "schemas",
         "second_microsecond",
@@ -976,20 +1023,28 @@ RESERVED_WORDS = set(
         "separator",
         "set",
         "show",
+        "signal",
+        "slow",  # 5.5
         "smallint",
+        "soname",
         "spatial",
         "specific",
         "sql",
-        "sqlexception",
-        "sqlstate",
-        "sqlwarning",
+        "sql_after_gtids",
+        "sql_before_gtids",  # 5.6
         "sql_big_result",
         "sql_calc_found_rows",
         "sql_small_result",
+        "sqlexception",
+        "sqlstate",
+        "sqlwarning",
         "ssl",
         "starting",
+        "stored",
         "straight_join",
+        "system",
         "table",
+        "tables",  # 4.1
         "terminated",
         "then",
         "tinyblob",
@@ -1016,70 +1071,17 @@ RESERVED_WORDS = set(
         "varchar",
         "varcharacter",
         "varying",
+        "virtual",  # 5.7
         "when",
         "where",
         "while",
+        "window",  # 8.0
         "with",
         "write",
         "x509",
         "xor",
         "year_month",
         "zerofill",  # 5.0
-        "columns",
-        "fields",
-        "privileges",
-        "soname",
-        "tables",  # 4.1
-        "accessible",
-        "linear",
-        "master_ssl_verify_server_cert",
-        "range",
-        "read_only",
-        "read_write",  # 5.1
-        "general",
-        "ignore_server_ids",
-        "master_heartbeat_period",
-        "maxvalue",
-        "resignal",
-        "signal",
-        "slow",  # 5.5
-        "get",
-        "io_after_gtids",
-        "io_before_gtids",
-        "master_bind",
-        "one_shot",
-        "partition",
-        "sql_after_gtids",
-        "sql_before_gtids",  # 5.6
-        "generated",
-        "optimizer_costs",
-        "stored",
-        "virtual",  # 5.7
-        "admin",
-        "cume_dist",
-        "empty",
-        "except",
-        "first_value",
-        "grouping",
-        "function",
-        "groups",
-        "json_table",
-        "last_value",
-        "nth_value",
-        "ntile",
-        "of",
-        "over",
-        "percent_rank",
-        "persist",
-        "persist_only",
-        "rank",
-        "recursive",
-        "role",
-        "row",
-        "rows",
-        "row_number",
-        "system",
-        "window",  # 8.0
     ]
 )
 
@@ -1203,17 +1205,63 @@ class MySQLCompiler(compiler.SQLCompiler):
     def visit_sysdate_func(self, fn, **kw):
         return "SYSDATE()"
 
-    def visit_json_getitem_op_binary(self, binary, operator, **kw):
-        return "JSON_EXTRACT(%s, %s)" % (
+    def _render_json_extract_from_binary(self, binary, operator, **kw):
+        # note we are intentionally calling upon the process() calls in the
+        # order in which they appear in the SQL String as this is used
+        # by positional parameter rendering
+
+        if binary.type._type_affinity is sqltypes.JSON:
+            return "JSON_EXTRACT(%s, %s)" % (
+                self.process(binary.left, **kw),
+                self.process(binary.right, **kw),
+            )
+
+        # for non-JSON, MySQL doesn't handle JSON null at all so it has to
+        # be explicit
+        case_expression = "CASE JSON_EXTRACT(%s, %s) WHEN 'null' THEN NULL" % (
             self.process(binary.left, **kw),
             self.process(binary.right, **kw),
         )
 
+        if binary.type._type_affinity is sqltypes.Integer:
+            type_expression = (
+                "ELSE CAST(JSON_EXTRACT(%s, %s) AS SIGNED INTEGER)"
+                % (
+                    self.process(binary.left, **kw),
+                    self.process(binary.right, **kw),
+                )
+            )
+        elif binary.type._type_affinity is sqltypes.Numeric:
+            # FLOAT / REAL not added in MySQL til 8.0.17
+            type_expression = (
+                "ELSE CAST(JSON_EXTRACT(%s, %s) AS DECIMAL(10, 6))"
+                % (
+                    self.process(binary.left, **kw),
+                    self.process(binary.right, **kw),
+                )
+            )
+        elif binary.type._type_affinity is sqltypes.Boolean:
+            # the NULL handling is particularly weird with boolean, so
+            # explicitly return true/false constants
+            type_expression = "WHEN true THEN true ELSE false"
+        elif binary.type._type_affinity is sqltypes.String:
+            # this fails with a JSON value that's a four byte unicode
+            # string.  SQLite has the same problem at the moment
+            type_expression = "ELSE JSON_UNQUOTE(JSON_EXTRACT(%s, %s))" % (
+                self.process(binary.left, **kw),
+                self.process(binary.right, **kw),
+            )
+        else:
+            # other affinity....this is not expected right now
+            type_expression = "ELSE JSON_EXTRACT(%s, %s)"
+
+        return case_expression + " " + type_expression + " END"
+
+    def visit_json_getitem_op_binary(self, binary, operator, **kw):
+        return self._render_json_extract_from_binary(binary, operator, **kw)
+
     def visit_json_path_getitem_op_binary(self, binary, operator, **kw):
-        return "JSON_EXTRACT(%s, %s)" % (
-            self.process(binary.left, **kw),
-            self.process(binary.right, **kw),
-        )
+        return self._render_json_extract_from_binary(binary, operator, **kw)
 
     def visit_on_duplicate_key_update(self, on_duplicate, **kw):
         if on_duplicate._parameter_ordering:
@@ -1230,15 +1278,15 @@ class MySQLCompiler(compiler.SQLCompiler):
                 c for c in self.statement.table.c if c.key not in ordered_keys
             ]
         else:
-            # traverse in table column order
             cols = self.statement.table.c
 
         clauses = []
-        for column in cols:
-            val = on_duplicate.update.get(column.key)
-            if val is None:
-                continue
-            elif elements._is_literal(val):
+        # traverses through all table columns to preserve table column order
+        for column in (col for col in cols if col.key in on_duplicate.update):
+
+            val = on_duplicate.update[column.key]
+
+            if elements._is_literal(val):
                 val = elements.BindParameter(None, val, type_=column.type)
                 value_text = self.process(val.self_group(), use_schema=False)
             elif isinstance(val, elements.BindParameter) and val.type._isnull:
@@ -1499,7 +1547,13 @@ class MySQLDDLCompiler(compiler.DDLCompiler):
             ),
         ]
 
-        is_timestamp = isinstance(column.type, sqltypes.TIMESTAMP)
+        if column.computed is not None:
+            colspec.append(self.process(column.computed))
+
+        is_timestamp = isinstance(
+            column.type._unwrapped_dialect_impl(self.dialect),
+            sqltypes.TIMESTAMP,
+        )
 
         if not column.nullable:
             colspec.append("NOT NULL")
@@ -2207,13 +2261,20 @@ class MySQLDialect(default.DefaultDialect):
         if util.py3k and isinstance(val, bytes):
             val = val.decode()
 
+        return self._parse_server_version(val)
+
+    def _parse_server_version(self, val):
         version = []
         r = re.compile(r"[.\-]")
         for n in r.split(val):
             try:
                 version.append(int(n))
             except ValueError:
-                version.append(n)
+                mariadb = re.match(r"(.*)(MariaDB)(.*)", n)
+                if mariadb:
+                    version.extend(g for g in mariadb.groups() if g)
+                else:
+                    version.append(n)
         return tuple(version)
 
     def do_commit(self, dbapi_connection):
@@ -2372,7 +2433,7 @@ class MySQLDialect(default.DefaultDialect):
 
         default.DefaultDialect.initialize(self, connection)
 
-        self._needs_correct_for_88718 = (
+        self._needs_correct_for_88718_96365 = (
             not self._is_mariadb and self.server_version_info >= (8,)
         )
 
@@ -2545,16 +2606,20 @@ class MySQLDialect(default.DefaultDialect):
             }
             fkeys.append(fkey_d)
 
-        if self._needs_correct_for_88718:
-            self._correct_for_mysql_bug_88718(fkeys, connection)
+        if self._needs_correct_for_88718_96365:
+            self._correct_for_mysql_bugs_88718_96365(fkeys, connection)
 
         return fkeys
 
-    def _correct_for_mysql_bug_88718(self, fkeys, connection):
+    def _correct_for_mysql_bugs_88718_96365(self, fkeys, connection):
         # Foreign key is always in lower case (MySQL 8.0)
         # https://bugs.mysql.com/bug.php?id=88718
         # issue #4344 for SQLAlchemy
 
+        # table name also for MySQL 8.0
+        # https://bugs.mysql.com/bug.php?id=96365
+        # issue #4751 for SQLAlchemy
+
         # for lower_case_table_names=2, information_schema.columns
         # preserves the original table/schema casing, but SHOW CREATE
         # TABLE does not.   this problem is not in lower_case_table_names=1,
@@ -2609,19 +2674,24 @@ class MySQLDialect(default.DefaultDialect):
             # is necessary
             d = defaultdict(dict)
             for schema, tname, cname in correct_for_wrong_fk_case:
+                d[(lower(schema), lower(tname))]["SCHEMANAME"] = schema
+                d[(lower(schema), lower(tname))]["TABLENAME"] = tname
                 d[(lower(schema), lower(tname))][cname.lower()] = cname
 
             for fkey in fkeys:
+                rec = d[
+                    (
+                        lower(fkey["referred_schema"] or default_schema_name),
+                        lower(fkey["referred_table"]),
+                    )
+                ]
+
+                fkey["referred_table"] = rec["TABLENAME"]
+                if fkey["referred_schema"] is not None:
+                    fkey["referred_schema"] = rec["SCHEMANAME"]
+
                 fkey["referred_columns"] = [
-                    d[
-                        (
-                            lower(
-                                fkey["referred_schema"] or default_schema_name
-                            ),
-                            lower(fkey["referred_table"]),
-                        )
-                    ][col.lower()]
-                    for col in fkey["referred_columns"]
+                    rec[col.lower()] for col in fkey["referred_columns"]
                 ]
 
     @reflection.cache
