@@ -1,5 +1,5 @@
 # testing/requirements.py
-# Copyright (C) 2005-2018 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -26,7 +26,6 @@ class Requirements(object):
 
 
 class SuiteRequirements(Requirements):
-
     @property
     def create_table(self):
         """target platform can emit basic CreateTable DDL."""
@@ -68,8 +67,8 @@ class SuiteRequirements(Requirements):
         # somehow only_if([x, y]) isn't working here, negation/conjunctions
         # getting confused.
         return exclusions.only_if(
-            lambda: self.on_update_cascade.enabled or
-            self.deferrable_fks.enabled
+            lambda: self.on_update_cascade.enabled
+            or self.deferrable_fks.enabled
         )
 
     @property
@@ -112,6 +111,15 @@ class SuiteRequirements(Requirements):
 
         return exclusions.open()
 
+    @property
+    def sql_expression_limit_offset(self):
+        """target database can render LIMIT and/or OFFSET with a complete
+        SQL expression, such as one that uses the addition operator.
+        parameter
+        """
+
+        return exclusions.open()
+
     @property
     def parens_in_union_contained_select_w_limit_offset(self):
         """Target database must support parenthesized SELECT in UNION
@@ -144,6 +152,12 @@ class SuiteRequirements(Requirements):
 
         return exclusions.closed()
 
+    @property
+    def nullable_booleans(self):
+        """Target database allows boolean columns to store NULL."""
+
+        return exclusions.open()
+
     @property
     def nullsordering(self):
         """Target backends that support nulls ordering."""
@@ -158,6 +172,14 @@ class SuiteRequirements(Requirements):
         """
         return exclusions.closed()
 
+    @property
+    def standalone_null_binds_whereclause(self):
+        """target database/driver supports bound parameters with NULL in the
+        WHERE clause, in situations where it has to be typed.
+
+        """
+        return exclusions.open()
+
     @property
     def intersect(self):
         """Target database must support INTERSECT or equivalent."""
@@ -179,10 +201,19 @@ class SuiteRequirements(Requirements):
 
         return exclusions.closed()
 
+    @property
+    def ctes_with_update_delete(self):
+        """target database supports CTES that ride on top of a normal UPDATE
+        or DELETE statement which refers to the CTE in a correlated subquery.
+
+        """
+
+        return exclusions.closed()
+
     @property
     def ctes_on_dml(self):
         """target database supports CTES which consist of INSERT, UPDATE
-        or DELETE"""
+        or DELETE *within* the CTE, e.g. WITH x AS (UPDATE....)"""
 
         return exclusions.closed()
 
@@ -222,22 +253,23 @@ class SuiteRequirements(Requirements):
     def sane_rowcount(self):
         return exclusions.skip_if(
             lambda config: not config.db.dialect.supports_sane_rowcount,
-            "driver doesn't support 'sane' rowcount"
+            "driver doesn't support 'sane' rowcount",
         )
 
     @property
     def sane_multi_rowcount(self):
         return exclusions.fails_if(
             lambda config: not config.db.dialect.supports_sane_multi_rowcount,
-            "driver %(driver)s %(doesnt_support)s 'sane' multi row count"
+            "driver %(driver)s %(doesnt_support)s 'sane' multi row count",
         )
 
     @property
     def sane_rowcount_w_returning(self):
         return exclusions.fails_if(
-            lambda config:
-                not config.db.dialect.supports_sane_rowcount_returning,
-            "driver doesn't support 'sane' rowcount when returning is on"
+            lambda config: not (
+                config.db.dialect.supports_sane_rowcount_returning
+            ),
+            "driver doesn't support 'sane' rowcount when returning is on",
         )
 
     @property
@@ -246,9 +278,9 @@ class SuiteRequirements(Requirements):
         INSERT DEFAULT VALUES or equivalent."""
 
         return exclusions.only_if(
-            lambda config: config.db.dialect.supports_empty_insert or
-            config.db.dialect.supports_default_values,
-            "empty inserts not supported"
+            lambda config: config.db.dialect.supports_empty_insert
+            or config.db.dialect.supports_default_values,
+            "empty inserts not supported",
         )
 
     @property
@@ -263,7 +295,7 @@ class SuiteRequirements(Requirements):
 
         return exclusions.only_if(
             lambda config: config.db.dialect.implicit_returning,
-            "%(database)s %(does_support)s 'returning'"
+            "%(database)s %(does_support)s 'returning'",
         )
 
     @property
@@ -288,7 +320,7 @@ class SuiteRequirements(Requirements):
 
         return exclusions.skip_if(
             lambda config: not config.db.dialect.requires_name_normalize,
-            "Backend does not require denormalized names."
+            "Backend does not require denormalized names.",
         )
 
     @property
@@ -298,7 +330,7 @@ class SuiteRequirements(Requirements):
 
         return exclusions.skip_if(
             lambda config: not config.db.dialect.supports_multivalues_insert,
-            "Backend does not support multirow inserts."
+            "Backend does not support multirow inserts.",
         )
 
     @property
@@ -342,31 +374,53 @@ class SuiteRequirements(Requirements):
 
         return exclusions.closed()
 
+    @property
+    def cross_schema_fk_reflection(self):
+        """target system must support reflection of inter-schema foreign keys
+
+        """
+        return exclusions.closed()
+
+    @property
+    def implicit_default_schema(self):
+        """target system has a strong concept of 'default' schema that can
+           be referred to implicitly.
+
+           basically, PostgreSQL.
+
+        """
+        return exclusions.closed()
+
     @property
     def server_side_cursors(self):
         """Target dialect must support server side cursors."""
 
-        return exclusions.only_if([
-            lambda config: config.db.dialect.supports_server_side_cursors
-        ], "no server side cursors support")
+        return exclusions.only_if(
+            [lambda config: config.db.dialect.supports_server_side_cursors],
+            "no server side cursors support",
+        )
 
     @property
     def sequences(self):
         """Target database must support SEQUENCEs."""
 
-        return exclusions.only_if([
-            lambda config: config.db.dialect.supports_sequences
-        ], "no sequence support")
+        return exclusions.only_if(
+            [lambda config: config.db.dialect.supports_sequences],
+            "no sequence support",
+        )
 
     @property
     def sequences_optional(self):
         """Target database supports sequences, but also optionally
         as a means of generating new PK values."""
 
-        return exclusions.only_if([
-            lambda config: config.db.dialect.supports_sequences and
-            config.db.dialect.sequences_optional
-        ], "no sequence support, or sequences not optional")
+        return exclusions.only_if(
+            [
+                lambda config: config.db.dialect.supports_sequences
+                and config.db.dialect.sequences_optional
+            ],
+            "no sequence support, or sequences not optional",
+        )
 
     @property
     def reflects_pk_names(self):
@@ -439,6 +493,11 @@ class SuiteRequirements(Requirements):
     def index_reflection(self):
         return exclusions.open()
 
+    @property
+    def indexes_with_expressions(self):
+        """target database supports CREATE INDEX against SQL expressions."""
+        return exclusions.closed()
+
     @property
     def unique_constraint_reflection(self):
         """target dialect supports reflection of unique constraints"""
@@ -593,6 +652,10 @@ class SuiteRequirements(Requirements):
 
         return self.json_type
 
+    @property
+    def json_index_supplementary_unicode_element(self):
+        return exclusions.open()
+
     @property
     def precision_numerics_general(self):
         """target backend has general support for moderately high-precision
@@ -619,6 +682,27 @@ class SuiteRequirements(Requirements):
         """
         return exclusions.closed()
 
+    @property
+    def implicit_decimal_binds(self):
+        """target backend will return a selected Decimal as a Decimal, not
+        a string.
+
+        e.g.::
+
+            expr = decimal.Decimal("15.7563")
+
+            value = e.scalar(
+                select([literal(expr)])
+            )
+
+            assert value == expr
+
+        See :ticket:`4036`
+
+        """
+
+        return exclusions.open()
+
     @property
     def nested_aggregates(self):
         """target database can select an aggregate from a subquery that's
@@ -692,6 +776,13 @@ class SuiteRequirements(Requirements):
 
         return exclusions.open()
 
+    @property
+    def expressions_against_unbounded_text(self):
+        """target database supports use of an unbounded textual field in a
+        WHERE clause."""
+
+        return exclusions.open()
+
     @property
     def selectone(self):
         """target driver must support the literal statement 'select 1'"""
@@ -721,16 +812,17 @@ class SuiteRequirements(Requirements):
 
     @property
     def update_where_target_in_subquery(self):
-        """Target must support UPDATE where the same table is present in a
-        subquery in the WHERE clause.
+        """Target must support UPDATE (or DELETE) where the same table is
+        present in a subquery in the WHERE clause.
 
         This is an ANSI-standard syntax that apparently MySQL can't handle,
-        such as:
+        such as::
+
+            UPDATE documents SET flag=1 WHERE documents.title IN
+                (SELECT max(documents.title) AS title
+                    FROM documents GROUP BY documents.user_id
+                )
 
-        UPDATE documents SET flag=1 WHERE documents.title IN
-            (SELECT max(documents.title) AS title
-                FROM documents GROUP BY documents.user_id
-            )
         """
         return exclusions.open()
 
@@ -752,6 +844,16 @@ class SuiteRequirements(Requirements):
         """
         return exclusions.closed()
 
+    @property
+    def order_by_col_from_union(self):
+        """target database supports ordering by a column from a SELECT
+        inside of a UNION
+
+        E.g.  (SELECT id, ...) UNION (SELECT id, ...) ORDER BY id
+
+        """
+        return exclusions.open()
+
     @property
     def order_by_label_with_expression(self):
         """target backend supports ORDER BY a column label within an
@@ -796,6 +898,13 @@ class SuiteRequirements(Requirements):
         """
         return exclusions.open()
 
+    @property
+    def independent_connections(self):
+        """
+        Target must support simultaneous, independent database connections.
+        """
+        return exclusions.open()
+
     @property
     def skip_mysql_on_windows(self):
         """Catchall for a large variety of MySQL on Windows failures"""
@@ -811,7 +920,8 @@ class SuiteRequirements(Requirements):
 
         """
         return exclusions.skip_if(
-            lambda config: config.options.low_connections)
+            lambda config: config.options.low_connections
+        )
 
     @property
     def timing_intensive(self):
@@ -829,37 +939,37 @@ class SuiteRequirements(Requirements):
         """
         return exclusions.skip_if(
             lambda config: util.py3k and config.options.has_coverage,
-            "Stability issues with coverage + py3k"
+            "Stability issues with coverage + py3k",
         )
 
     @property
     def python2(self):
         return exclusions.skip_if(
             lambda: sys.version_info >= (3,),
-            "Python version 2.xx is required."
+            "Python version 2.xx is required.",
         )
 
     @property
     def python3(self):
         return exclusions.skip_if(
-            lambda: sys.version_info < (3,),
-            "Python version 3.xx is required."
+            lambda: sys.version_info < (3,), "Python version 3.xx is required."
         )
 
     @property
     def cpython(self):
         return exclusions.only_if(
-            lambda: util.cpython,
-            "cPython interpreter needed"
+            lambda: util.cpython, "cPython interpreter needed"
         )
 
     @property
     def non_broken_pickle(self):
         from sqlalchemy.util import pickle
+
         return exclusions.only_if(
-            lambda: not util.pypy and pickle.__name__ == 'cPickle'
-                or sys.version_info >= (3, 2),
-            "Needs cPickle+cPython or newer Python 3 pickle"
+            lambda: not util.pypy
+            and pickle.__name__ == "cPickle"
+            or sys.version_info >= (3, 2),
+            "Needs cPickle+cPython or newer Python 3 pickle",
         )
 
     @property
@@ -880,7 +990,7 @@ class SuiteRequirements(Requirements):
         """
         return exclusions.skip_if(
             lambda config: config.options.has_coverage,
-            "Issues observed when coverage is enabled"
+            "Issues observed when coverage is enabled",
         )
 
     def _has_mysql_on_windows(self, config):
@@ -901,15 +1011,21 @@ class SuiteRequirements(Requirements):
 
     def _has_sqlite(self):
         from sqlalchemy import create_engine
+
         try:
-            create_engine('sqlite://')
+            create_engine("sqlite://")
             return True
         except ImportError:
             return False
 
     def _has_cextensions(self):
         try:
-            from sqlalchemy import cresultproxy, cprocessors
+            from sqlalchemy import cresultproxy, cprocessors  # noqa
+
             return True
         except ImportError:
             return False
+
+    @property
+    def computed_columns(self):
+        return exclusions.closed()
